=== modified file 'NEWS'
--- NEWS	2007-12-14 08:47:17 +0000
+++ NEWS	2007-12-14 15:38:28 +0000
@@ -20,6 +20,8 @@
    * Merge directives now fetch prerequisites from the target branch if
      needed.  (Aaron Bentley)
 
+   * ``reconfigure`` can now convert from repositories.  (Aaron Bentley)
+
   BUGFIXES:
 
    * Fix Makefile rules for doc generation. (Ian Clatworthy, #175207)
@@ -35,6 +37,9 @@
      read the remaining bytes by chunks to avoid overflowing network buffers.
      (Vincent Ladeuil, #175886)
 
+   * ``reconfigure`` is able to convert trees into lightweight checkouts.
+     (Aaron Bentley)
+
   INTERNALS:
 
   API BREAKS:

=== modified file 'bzrlib/reconfigure.py'
--- bzrlib/reconfigure.py	2007-11-27 19:37:36 +0000
+++ bzrlib/reconfigure.py	2007-12-13 16:38:54 +0000
@@ -62,8 +62,6 @@
 
         :param bzrdir: The bzrdir to reconfigure
         :raise errors.AlreadyBranch: if bzrdir is already a branch
-        :raise errors.ReconfigurationNotSupported: if bzrdir does not contain
-            a branch or branch reference
         """
         reconfiguration = Reconfigure(bzrdir)
         reconfiguration._plan_changes(want_tree=False, want_branch=True,
@@ -78,8 +76,6 @@
 
         :param bzrdir: The bzrdir to reconfigure
         :raise errors.AlreadyTree: if bzrdir is already a tree
-        :raise errors.ReconfigurationNotSupported: if bzrdir does not contain
-            a branch or branch reference
         """
         reconfiguration = Reconfigure(bzrdir)
         reconfiguration._plan_changes(want_tree=True, want_branch=True,
@@ -95,8 +91,6 @@
         :param bzrdir: The bzrdir to reconfigure
         :param bound_location: The location the checkout should be bound to.
         :raise errors.AlreadyCheckout: if bzrdir is already a checkout
-        :raise errors.ReconfigurationNotSupported: if bzrdir does not contain
-            a branch or branch reference
         """
         reconfiguration = Reconfigure(bzrdir, bound_location)
         reconfiguration._plan_changes(want_tree=True, want_branch=True,
@@ -113,8 +107,6 @@
         :param bound_location: The location the checkout should be bound to.
         :raise errors.AlreadyLightweightCheckout: if bzrdir is already a
             lightweight checkout
-        :raise errors.ReconfigurationNotSupported: if bzrdir does not contain
-            a branch or branch reference
         """
         reconfiguration = klass(bzrdir, reference_location)
         reconfiguration._plan_changes(want_tree=True, want_branch=False,
@@ -130,17 +122,14 @@
             raise errors.ReconfigurationNotSupported(self.bzrdir)
         if want_branch and want_reference:
             raise errors.ReconfigurationNotSupported(self.bzrdir)
-        if (want_branch or want_reference) and (self.local_branch is None and
-                                                self.referenced_branch
-                                                is None):
-            raise errors.ReconfigurationNotSupported(self.bzrdir)
         if self.repository is None:
             if not want_reference:
                 self._create_repository = True
         else:
             if want_reference and (self.repository.bzrdir.root_transport.base
                                    == self.bzrdir.root_transport.base):
-                self._destroy_repository = True
+                if not self.repository.is_shared():
+                    self._destroy_repository = True
         if self.referenced_branch is None:
             if want_reference:
                 self._create_reference = True
@@ -168,7 +157,8 @@
         """Return True if changes are planned, False otherwise"""
         return (self._unbind or self._bind or self._destroy_tree
                 or self._create_tree or self._destroy_reference
-                or self._create_branch or self._create_repository)
+                or self._create_branch or self._create_repository
+                or self._create_reference)
 
     def _check(self):
         """Raise if reconfiguration would destroy local changes"""
@@ -223,18 +213,20 @@
             repo = self.bzrdir.create_repository()
         else:
             repo = self.repository
-        if self._create_branch:
+        if self._create_branch and self.referenced_branch is not None:
             repo.fetch(self.referenced_branch.repository,
                        self.referenced_branch.last_revision())
+        last_revision_info = None
         if self._destroy_reference:
-            reference_info = self.referenced_branch.last_revision_info()
+            last_revision_info = self.referenced_branch.last_revision_info()
             self.bzrdir.destroy_branch()
         if self._destroy_branch:
-            reference_info = self.local_branch.last_revision_info()
+            last_revision_info = self.local_branch.last_revision_info()
             self.bzrdir.destroy_branch()
         if self._create_branch:
             local_branch = self.bzrdir.create_branch()
-            local_branch.set_last_revision_info(*reference_info)
+            if last_revision_info is not None:
+                local_branch.set_last_revision_info(*last_revision_info)
         else:
             local_branch = self.local_branch
         if self._create_reference:

=== modified file 'bzrlib/tests/test_reconfigure.py'
--- bzrlib/tests/test_reconfigure.py	2007-11-29 17:32:23 +0000
+++ bzrlib/tests/test_reconfigure.py	2007-12-13 16:38:54 +0000
@@ -18,6 +18,7 @@
     branch as _mod_branch,
     errors,
     reconfigure,
+    repository,
     tests,
     workingtree,
     )
@@ -49,8 +50,8 @@
 
     def test_repo_to_branch(self):
         repo = self.make_repository('repo')
-        self.assertRaises(errors.ReconfigurationNotSupported,
-                          reconfigure.Reconfigure.to_branch, repo.bzrdir)
+        reconfiguration = reconfigure.Reconfigure.to_branch(repo.bzrdir)
+        reconfiguration.apply()
 
     def test_checkout_to_branch(self):
         branch = self.make_branch('branch')
@@ -146,6 +147,24 @@
                                                               parent.base)
         reconfiguration.apply()
 
+    def test_tree_to_lightweight_checkout(self):
+        # A tree with no related branches and no supplied bind location cannot
+        # become a checkout
+        parent = self.make_branch('parent')
+
+        tree = self.make_branch_and_tree('tree')
+        reconfiguration = reconfigure.Reconfigure.to_lightweight_checkout(
+            tree.bzrdir)
+        self.assertRaises(errors.NoBindLocation, reconfiguration.apply)
+        # setting a parent allows it to become a checkout
+        tree.branch.set_parent(parent.base)
+        reconfiguration.apply()
+        # supplying a location allows it to become a checkout
+        tree2 = self.make_branch_and_tree('tree2')
+        reconfiguration = reconfigure.Reconfigure.to_lightweight_checkout(
+            tree2.bzrdir, parent.base)
+        reconfiguration.apply()
+
     def test_checkout_to_checkout(self):
         parent = self.make_branch('parent')
         checkout = parent.create_checkout('checkout')
@@ -187,3 +206,31 @@
         self.assertRaises(errors.AlreadyLightweightCheckout,
                           reconfigure.Reconfigure.to_lightweight_checkout,
                           checkout.bzrdir)
+
+    def test_repo_to_tree(self):
+        repo = self.make_repository('repo')
+        reconfiguration = reconfigure.Reconfigure.to_tree(repo.bzrdir)
+        reconfiguration.apply()
+        workingtree.WorkingTree.open('repo')
+
+    def test_shared_repo_to_lightweight_checkout(self):
+        repo = self.make_repository('repo', shared=True)
+        reconfiguration = reconfigure.Reconfigure.to_lightweight_checkout(
+            repo.bzrdir)
+        self.assertRaises(errors.NoBindLocation, reconfiguration.apply)
+        branch = self.make_branch('branch')
+        reconfiguration = reconfigure.Reconfigure.to_lightweight_checkout(
+            repo.bzrdir, 'branch')
+        reconfiguration.apply()
+        workingtree.WorkingTree.open('repo')
+        repository.Repository.open('repo')
+
+    def test_unshared_repo_to_lightweight_checkout(self):
+        repo = self.make_repository('repo', shared=False)
+        branch = self.make_branch('branch')
+        reconfiguration = reconfigure.Reconfigure.to_lightweight_checkout(
+            repo.bzrdir, 'branch')
+        reconfiguration.apply()
+        workingtree.WorkingTree.open('repo')
+        self.assertRaises(errors.NoRepositoryPresent,
+                          repository.Repository.open, 'repo')

